/******************************************************************************
* File Name: main.c
*
* Version: 2.00
*
* Description: This code example demonstrates how to implement a low-power 
*              CapSense button with an average current consumption of 5uA
*              per button. 
*
* Related Document: CE210488 LP CapSense Buttons.pdf
*
* Hardware Dependency: See code example CE210488
*
******************************************************************************
* Copyright (2016), Cypress Semiconductor Corporation.
******************************************************************************
* This software, including source code, documentation and related materials
* ("Software") is owned by Cypress Semiconductor Corporation (Cypress) and is
* protected by and subject to worldwide patent protection (United States and 
* foreign), United States copyright laws and international treaty provisions. 
* Cypress hereby grants to licensee a personal, non-exclusive, non-transferable
* license to copy, use, modify, create derivative works of, and compile the 
* Cypress source code and derivative works for the sole purpose of creating 
* custom software in support of licensee product, such licensee product to be
* used only in conjunction with Cypress's integrated circuit as specified in the
* applicable agreement. Any reproduction, modification, translation, compilation,
* or representation of this Software except as specified above is prohibited 
* without the express written permission of Cypress.
* 
* Disclaimer: THIS SOFTWARE IS PROVIDED AS-IS, WITH NO WARRANTY OF ANY KIND, 
* EXPRESS OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, NONINFRINGEMENT, IMPLIED 
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
* Cypress reserves the right to make changes to the Software without notice. 
* Cypress does not assume any liability arising out of the application or use
* of Software or any product or circuit described in the Software. Cypress does
* not authorize its products for use as critical components in any products 
* where a malfunction or failure may reasonably be expected to result in 
* significant injury or death ("ACTIVE Risk Product"). By including Cypress's 
* product in a ACTIVE Risk Product, the manufacturer of such system or application
* assumes all risk of such use and in doing so indemnifies Cypress against all
* liability. Use of this Software may be limited by and subject to the applicable
* Cypress software license agreement.
*****************************************************************************/
/*******************************************************************************
* Theory of Operation: This example code demonstrates low power CapSense button 
* functionality. It will help user to learn how to set parameters in their design 
* for low power CapSense button application. Left button controls the RGB LED color 
* in the order R->G->B->R. Right button controls the brightness of the RGB LED
* in the order Min->Mid->Max. 

* Note: The time of LOOP_TIME_FASTSCANMODE and LOOP_TIME_SLOWSCANMODE are dependent   
* on ILO accuracy. With ILO, the accuracy of the time can vary as much as 60 percent.  
* ILO trim method is used to improve the ILO accuracy to 10 percent.
*******************************************************************************/

/*******************************************************************************
*   Included Headers
*******************************************************************************/
#include "project.h"

/* Set the struct alignment mode */
#pragma pack(1)

#define CAPS_SENSOR_NUMBER          CapSense_TOTAL_SENSORS
#define CAPS_TUNER                  0u

typedef struct I2CRamBuffer
{
    uint8  u8_sensorNumbers;                      // addr 0       
    uint16 u16_cmodValue;                         // 1-2            
    uint8  u8_cpValue[CapSense_TOTAL_SENSORS];    // 3, 4,        
}I2CRamBuffer;	

#if !CAPS_TUNER
struct I2CRamBuffer MyI2CRegs;
#endif

int main()
{	

    #if !CAPS_TUNER
        uint16 i;
    #endif
    
    /* Enable global interrupts. */ 
    CyGlobalIntEnable;    
    
    #if !CAPS_TUNER

    MyI2CRegs.u8_sensorNumbers = CAPS_SENSOR_NUMBER;       
    MyI2CRegs.u16_cmodValue = 0x0000u;
    
    for(i=0; i<CapSense_TOTAL_SENSORS; i++)
    {        
        MyI2CRegs.u8_cpValue[i] = 0x00u;
    }    
    #endif
    
    /* Set up communication data buffer to CapSense data structure to 
    *  expose to I2C master at primary slave address request		
    */
    #if CAPS_TUNER
        EZI2C_EzI2CSetBuffer1(sizeof(CapSense_dsRam), sizeof(CapSense_dsRam),\
                                (uint8 *)&CapSense_dsRam);
    #endif
    
    /* set Customized EZI2C buffer */
     #if !CAPS_TUNER
        EZI2C_EzI2CSetBuffer1(sizeof(MyI2CRegs), 2u, (void *)&MyI2CRegs);
    #endif
    
    /* Initialize I2C component for CapSense tuner */
    EZI2C_Start();
    
    /* Initialize the CapSense CSD block */
	CapSense_Start();
    CapSense_ScanAllWidgets(); /* Scan all widgets */
    while(CapSense_NOT_BUSY != CapSense_IsBusy()){}
    CapSense_FtInitialize();
    CapSense_InitializeAllBaselines();

    for(;;)
    {
        /*BIST Cp Cmod*/
        #if !CAPS_TUNER
            CapSense_Stop();            
            CapSense_Start();
            CapSense_ScanAllWidgets(); /* Scan all widgets */
            while(CapSense_NOT_BUSY != CapSense_IsBusy()){}
            CapSense_FtInitialize();
            CapSense_InitializeAllBaselines();
            
            /*Measure Cmod*/  
            MyI2CRegs.u16_cmodValue = (uint16)CapSense_GetExtCapCapacitance(CapSense_TST_CMOD_ID);

            /*Measure CP*/  
            for(i=0;i<CapSense_TOTAL_WIDGETS;i++)
            {
                MyI2CRegs.u8_cpValue[i] = (uint8)CapSense_GetSensorCapacitance (i, 0);
            }           
        #endif
            
        while(CapSense_NOT_BUSY != CapSense_IsBusy()){}
        CapSense_ProcessAllWidgets();
        
        #if CAPS_TUNER
            CapSense_RunTuner();
        #endif
        
        CapSense_ScanAllWidgets();

    }
}
